# -*- coding: utf-8 -*-
from dbaas_foreman.dbaas_api import DatabaseAsAServiceApi
from dbaas_foreman.foreman_provider import ForemanProvider


def get_foreman_provider(databaseinfra, credentials):
    dbaas_api = DatabaseAsAServiceApi(databaseinfra, credentials)
    return ForemanProvider(dbaas_api)



# -*- coding: utf-8 -*-


class FakeForeman(object):
    def __init__(self, url, auth, api_version):
        self.url = url,
        self.auth = auth,
        self.api_version = api_version
        self.last_call = []

    def do_request(self, method, params):
        self.last_call.append({'method': method, 'params': params})

        if any((1 for key, value in params.items() if 'id' in key and value.endswith('_fails'))):
            return {}
        elif method == 'puppetclasses.show':
            if 'id' in params:
                return {'id': '171'}

        return {'something': 'new'}

    def __getattr__(self, attr):
        return ForemanMethodHandler(attr, self)


class ForemanMethodHandler(object):
    def __init__(self, name, parent):
        self.name = name
        self.parent = parent

    def __getattr__(self, attr):
        def fn(*args, **kwargs):
            params = {}
            params['args'] = args
            params.update(kwargs)
            return self.parent.do_request(
                '{0}.{1}'.format(self.name, attr),
                params
            )
        return fn


class Host(object):
    def __init__(self, address, dns):
        self.address = address
        self.hostname = dns


class Engine(object):
    def __init__(self, name, version='0.0.0'):
        self.engine_type = EngineType(name)
        self.version = version


class EngineType(object):
    def __init__(self, name):
        self.name = name


class Plan(object):
    def __init__(self, is_ha):
        self.is_ha = is_ha


class Instance(object):
    def __init__(self, dns, hostname):
        self.address = hostname.address
        self.dns = dns
        self.hostname = hostname


class Driver(object):
    def __init__(self, databaseinfra):
        self.databaseinfra = databaseinfra

    def get_database_instances(self):
        return self.databaseinfra.instances

    def get_non_database_instances(self):
        return self.databaseinfra.instances


class DatabaseInfra(object):
    def __init__(self, instances, environment, plan, name):
        self.instances = instances
        self.environment = environment
        self.name = name
        self.engine = Engine(name)
        self.plan = plan

    def get_driver(self):
        if hasattr(self, 'driver'):
            return self.driver

        self.driver = Driver(self)
        return self.driver


class InstanceList(list):
    def all(self):
        return self


def set_up_databaseinfra(is_ha=True, name="fake"):
    instances = InstanceList()
    plan = Plan(is_ha)
    for n in range(1, 4):
        address = '10.10.10.1{}'.format(n)
        dns = 'myhost_{}'.format(n)
        host = Host(address, dns + '.com')

        instance = Instance(dns + '.database.com', host)
        instances.append(instance)

    return DatabaseInfra(instances, 'development', plan, name)


class FakeCredential(object):
        def __init__(self):
            self.user = ''
            self.password = ''
            self.endpoint = ''

        def get_parameter_by_name(self, name):
            return ''

        def get_parameters_by_group(self, group_name):
            return {}



# -*- coding: utf-8 -*-


class DatabaseAsAServiceApi(object):
    def __init__(self, databaseinfra, credentials):
        self.databaseinfra = databaseinfra
        self.credentials = credentials

    @property
    def user(self):
        return self.credentials.user

    @property
    def password(self):
        return self.credentials.password

    @property
    def endpoint(self):
        return self.credentials.endpoint

    @property
    def dscp_foreman_class(self):
        return self.credentials.get_parameter_by_name('dscp_foreman_class')

    @property
    def dscp_foreman_param_name(self):
        return self.credentials.get_parameter_by_name('dscp_foreman_param_name')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pytest
from dbaas_foreman.foreman_provider import ForemanProvider
from dbaas_foreman.dbaas_api import DatabaseAsAServiceApi
from dbaas_foreman import exceptions
from .factory import FakeForeman, set_up_databaseinfra, FakeCredential


def assert_last_call(foreman_provider, method, params, args=()):
    last_call = foreman_provider._foreman_client.last_call.pop()
    params.update({'args': args})

    assert last_call['method'] == method
    assert last_call['params'] == params


@pytest.fixture
def foreman_provider():
    databaseinfra = set_up_databaseinfra()
    dbaas_api = DatabaseAsAServiceApi(databaseinfra, FakeCredential())
    return ForemanProvider(
        dbaas_api=dbaas_api,
        foreman_client_class=FakeForeman
    )


def test_search_host_succesfuly(foreman_provider):
    foreman_provider._search_host('example01.does.not.exists.com')
    assert_last_call(
        foreman_provider, 'hosts.show',
        {'id': 'example01.does.not.exists.com'}
    )


def test_search_puppet_class_succesfuly(foreman_provider):
    foreman_provider._search_puppet_class('my_puppet_class')
    assert_last_call(
        foreman_provider, 'puppetclasses.show', {'id': 'my_puppet_class'}
    )


def test_add_parameter_to_host_succesfuly(foreman_provider):
    foreman_provider._add_parameter_to_host(
        'test_host', 'test_param_name', 'test_param_value'
    )

    assert_last_call(
        foreman_provider, 'hosts.parameters_create',
        {'host_id': 'test_host'},
        ({'name': 'test_param_name', 'value': 'test_param_value'},)
    )


def test_add_parameter_to_host_fails(foreman_provider):
    with pytest.raises(exceptions.HostParameterNotCreatedError):
        foreman_provider._add_parameter_to_host(
            'test_host_fails', 'test_param_name', 'test_param_value'
        )


def test_add_puppet_class_to_host_succesfuly(foreman_provider):
    foreman_provider._add_puppet_class_to_host(
        'test_host', 'my_puppet_class_with_id'
    )

    assert_last_call(
        foreman_provider, 'hosts.host_classes_create',
        {'host_id': 'test_host', 'puppetclass_id': '171'},
    )


def test_add_puppet_class_to_host_puppet_class_not_found(foreman_provider):
    with pytest.raises(exceptions.PuppetClassNotFound):
        foreman_provider._add_puppet_class_to_host(
            'test_host', 'my_puppet_class_with_id_fails'
        )


def test_add_puppet_class_to_host_puppet_class_not_created_error(foreman_provider):
    with pytest.raises(exceptions.HostPuppetClassNotCreatedError):
        foreman_provider._add_puppet_class_to_host(
            'test_host_fails', 'my_puppet_class_with_id'
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup

__author__ = 'Felippe da Motta Raposo'
__email__ = 'raposo.felippe@gmail.com'
__version__ = '0.1.9'


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt') as requirements_file:
    requirements = requirements_file.read()

with open('requirements_test.txt') as requirements_test_file:
    test_requirements = requirements_test_file.read()


setup(
    name='dbaas_foreman',
    version=__version__,
    description="DBaaS Foreman is a simple foreman api wrapper for DBaaS",
    long_description=readme + '\n\n' + history,
    author="Felippe da Motta Raposo",
    author_email='raposo.felippe@gmail.com',
    url='https://github.com/felippemr/dbaas_foreman',
    packages=[
        'dbaas_foreman',
    ],
    package_dir={'dbaas_foreman':
                 'dbaas_foreman'},
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='dbaas_foreman',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
